#!/usr/bin/env python

"""
BaseHTTPRequestHandler classes.
"""

import Generic
from Helpers.Request import MessageBodyStream
from Helpers.Auth import UserInfo
from cgi import FieldStorage
from StringIO import StringIO

class Transaction(Generic.Transaction):

    """
    BaseHTTPRequestHandler transaction interface.
    """

    def __init__(self, trans):

        """
        Initialise the transaction using the BaseHTTPRequestHandler instance
        'trans'.
        """

        self.trans = trans

        # Other attributes of interest in instances of this class.

        self.content_type = None
        self.response_code = 200
        self.content = StringIO()
        self.headers = {}

    def commit(self):

        """
        A special method, synchronising the transaction with framework-specific
        objects.
        """

        self.trans.send_response(self.response_code)
        if self.content_type is not None:
            self.trans.send_header("Content-Type", self.format_content_type(self.content_type))
        for header, value in self.headers.items():
            self.trans.send_header(self.format_header_value(header), self.format_header_value(value))
        self.trans.end_headers()
        self.content.seek(0)
        self.trans.wfile.write(self.content.read())

    # Request-related methods.

    def get_request_stream(self):

        """
        A framework-specific method which returns the request stream for
        the transaction.
        """

        return MessageBodyStream(self.trans.rfile, self.get_headers())

    def get_request_method(self):

        """
        A framework-specific method which gets the request method.
        """

        return self.trans.command

    def get_headers(self):

        """
        A framework-specific method which returns all request headers.
        """

        return self.trans.headers

    def get_header_values(self, key):

        """
        A framework-specific method which returns a list of all request header
        values associated with the given 'key'. Note that according to RFC 2616,
        'key' is treated as a case-insensitive string.
        """

        return self.convert_to_list(self.trans.headers.get(key))

    def get_content_type(self):

        """
        A framework-specific method which gets the content type specified on the
        request, along with the charset employed.
        """

        return self.parse_content_type(self.trans.headers.get("Content-type") or
            self.trans.headers.get("Content-Type"))

    def get_content_charsets(self):

        """
        Returns the character set preferences.
        """

        return self.parse_content_preferences(self.trans.headers["Accept-Charset"])

    def get_content_languages(self):

        """
        A framework-specific method which extracts language information from
        the transaction.
        """

        return self.parse_content_preferences(self.trans.headers["Accept-Language"])

    def get_path(self):

        """
        A framework-specific method which gets the entire path from the request.
        """

        return self.trans.path

    def get_path_info(self):

        """
        A framework-specific method which gets the "path info" (the part of the
        URL after the resource name handling the current request) from the
        request.
        """

        # NOTE: No attempt is made to deduce the "path info".

        return self.trans.path

    # Higher level request-related methods.

    def get_fields(self):

        """
        A framework-specific method which extracts the form fields from the
        transaction.
        """

        return FieldStorage(self.get_request_stream(), keep_blank_values=1)

    def get_user(self):

        """
        A framework-specific method which extracts user information from the
        transaction.
        """

        auth_header = self.get_headers().get("Authorization")
        if auth_header:
            return UserInfo(auth_header).username
        else:
            return None

    # Response-related methods.

    def get_response_stream(self):

        """
        A framework-specific method which returns the response stream for
        the transaction.
        """

        # Return a stream which is later emptied into the real stream.

        return self.content

    def get_response_code(self):

        """
        Get the response code associated with the transaction. If no response
        code is defined, None is returned.
        """

        return self.response_code

    def set_response_code(self, response_code):

        """
        Set the 'response_code' using a numeric constant defined in the HTTP
        specification.
        """

        self.response_code = response_code

    def set_header_value(self, header, value):

        """
        Set the HTTP 'header' with the given 'value'.
        """

        # The header is not written out immediately due to the buffering in use.

        self.headers[header] = value

    def set_content_type(self, content_type):

        """
        A framework-specific method which sets the 'content_type' for the
        response.
        """

        # The content type has to be written as a header, before actual content,
        # but after the response line. This means that some kind of buffering is
        # required. Hence, we don't write the header out immediately.

        self.content_type = content_type

# vim: tabstop=4 expandtab shiftwidth=4
